(function() {
	var ZeedhiDirectives = angular.module('ZeedhiDirectives');
	ZeedhiDirectives.directive('zhImageUploader', zhImageUploader);

	function zhImageUploader(templateManager, ZHPromise) {

		var linkFunction = function(scope, element, attrs) {
			var container = element.parent();
			var field = scope.field;
			var widget = field.widget;
			var filesPromises = [];

			field.fileList = [];
			element.shouldNotPreventDefaultOnClick = true;
			element.parent().find("ul").nonbounce();
			element.on('change', onFileSelected);

			function onFileSelected() {
				var files = this.files;

				if (files.length > 0) {
					_ensureCurrentRowPropertyExistence();
					_setCurrentRowAsChanged();
					_putFilesIntoPromises(files);
					_executeAllPromises();
				}

				this.value = "";
			}

			function _ensureCurrentRowPropertyExistence() {
				widget.currentRow = widget.currentRow || {};
				widget.currentRow[field.name] = widget.currentRow[field.name] || [];
			}

			function _setCurrentRowAsChanged() {
				widget.currentRow.changed = true;
			}

			function _putFilesIntoPromises(files) {
				for (var i in files) {
					if (files.hasOwnProperty(i) && files[i] instanceof File) {
						filesPromises.push(_buildFileData(files[i]));
					}
				}
			}

			function _buildFileData(file) {
				var deferred = ZHPromise.defer();
				var fileReader = new FileReader();

				fileReader.onload = function(event) {
					widget.currentRow[field.name] = [_addExtraDataToFile(file)];
					field.fileList = widget.currentRow[field.name];
					deferred.resolve();
				};

				fileReader.readAsDataURL(file);
				return deferred;
			}

			function _addExtraDataToFile(file) {
				return Util.extend(file, {
					b64File: event.target.result
				}, true);
			}

			function _executeAllPromises() {
				ZHPromise.all(filesPromises)
					.then(function() {
						_callAfterSelectFileEvent();
						field.validate(field.value());
						templateManager.updateTemplate();
					});
			}

			function _callAfterSelectFileEvent() {
				if (field.afterSelectFile) {
					field.afterSelectFile();
				}
			}
		};

		return {
			restrict: 'A',
			link: linkFunction
		};
	}
})();